package ch.zhaw.gartenverwaltung;

import ch.zhaw.gartenverwaltung.io.HardinessZoneNotSetException;
import ch.zhaw.gartenverwaltung.plantList.PlantListModel;
import ch.zhaw.gartenverwaltung.types.HardinessZone;
import ch.zhaw.gartenverwaltung.types.Plant;
import ch.zhaw.gartenverwaltung.types.Seasons;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableValue;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.geometry.Insets;
import javafx.scene.control.*;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.InputMethodEvent;
import javafx.scene.layout.VBox;

import java.io.IOException;
import java.net.URL;
import java.util.LinkedList;
import java.util.List;
import java.util.ResourceBundle;

public class PlantsController implements Initializable {
    private final PlantListModel plantListModel = new PlantListModel();
    private Plant selectedPlant = null;
    private final HardinessZone DEFAULT_HARDINESS_ZONE = HardinessZone.ZONE_8A;

    @FXML
    private VBox seasons;

    @FXML
    private VBox climate_zones;

    @FXML
    private Label description_plant;

    @FXML
    private ImageView img_plant;

    @FXML
    private ListView<Plant> list_plants;

    @FXML
    private Button saveToMyPlant_button;

    @FXML
    private TextField search_plants;


    @FXML
    void saveToMyPlant(ActionEvent event) {
        //ToDo model save selectedPlant to mySelectedPlant(IO)
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        fillPlantListWithDefaultValues();

        description_plant.setText("");
        saveToMyPlant_button.setDisable(true);

        createFilterSeasons();
        createFilterHardinessZone();
        lookForSelectedListEntry();
        try {
            viewFilteredListBySearch();
        } catch (HardinessZoneNotSetException | IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * update the ListView according to the plant list provided
     * Entry in ListView is plant name
     * @param list plantList which fill the ListView
     */
    private void fillListViewWithData(List<Plant> list) {
        clearListView();
        for (Plant plant : list) {
            list_plants.getItems().add(plant);
        }
        list_plants.setCellFactory(param -> new ListCell<Plant>() {
            @Override
            protected void updateItem(Plant plant, boolean empty) {
                super.updateItem(plant, empty);

                if (empty || plant == null || plant.name() == null) {
                    setText(null);
                } else {
                    setText(plant.name());
                }
            }
        });
    }

    private void viewFilteredListBySeason(Seasons season) throws HardinessZoneNotSetException, IOException {
        List<Plant> plantList = plantListModel.getFilteredPlantListBySaisonWithoutGrowthPhase(plantListModel.getCurrentZone(), season.getStartDate(), season.getEndDate());
        fillListViewWithData(plantList);
    }

    private void viewFilteredListBySearch() throws HardinessZoneNotSetException, IOException {
        search_plants.textProperty().addListener((observable, oldValue, newValue) -> {
            if (newValue.isEmpty()) {
                fillPlantListWithDefaultValues();
            }else {
                List<Plant> plantList = new LinkedList<>();
                try {
                    plantList = plantListModel.getFilteredPlantListByString(plantListModel.getCurrentZone(), newValue);
                } catch (HardinessZoneNotSetException | IOException e) {
                    e.printStackTrace();
                }
                fillListViewWithData(plantList);
            }
        });
    }

    private void fillPlantListWithDefaultValues() {
        List<Plant> plantList = new LinkedList<>();
        try {
            plantList = plantListModel.getPlantList(plantListModel.getCurrentZone());
        } catch (HardinessZoneNotSetException | IOException e) {
            e.printStackTrace();
        }
        fillListViewWithData(plantList);
    }

    private void createFilterHardinessZone() {
        ToggleGroup hardinessGroup = new ToggleGroup();
        for (HardinessZone zone : HardinessZone.values()) {
            RadioButton radioButton = new RadioButton(zone.name());
            radioButton.setToggleGroup(hardinessGroup);
            radioButton.setPadding(new Insets(0,0,10,0));
            if (zone.equals(DEFAULT_HARDINESS_ZONE)) {
                radioButton.setSelected(true);
            }
            radioButton.selectedProperty().addListener(new ChangeListener<Boolean>() {
                @Override
                public void changed(ObservableValue<? extends Boolean> observable, Boolean oldValue, Boolean newValue) {
                    plantListModel.setCurrentZone(zone);
                    fillPlantListWithDefaultValues();
                }
            });
            climate_zones.getChildren().add(radioButton);
        }
    }

    private void createFilterSeasons() {
        ToggleGroup seasonGroup = new ToggleGroup();
        for (Seasons season : Seasons.values()) {
            RadioButton radioButton = new RadioButton(season.name());
            radioButton.setToggleGroup(seasonGroup);
            radioButton.setPadding(new Insets(0,0,10,0));
            if (season.equals(Seasons.AllSEASONS)) {
                radioButton.setSelected(true);
            }
            radioButton.selectedProperty().addListener(new ChangeListener<Boolean>() {
                @Override
                public void changed(ObservableValue<? extends Boolean> observable, Boolean oldValue, Boolean newValue) {
                    if (season.equals(Seasons.AllSEASONS)) {
                        fillPlantListWithDefaultValues();
                    } else {
                        try {
                            viewFilteredListBySeason(season);
                        } catch (HardinessZoneNotSetException | IOException e) {
                            e.printStackTrace();
                        }
                    }

                }
            });
            seasons.getChildren().add(radioButton);
        }
    }

    /**
     * observes changes in the selectedProperty of ListView and updates the description label
     */
    private void lookForSelectedListEntry() {
        list_plants.getSelectionModel().selectedItemProperty().addListener(new ChangeListener<Plant>() {
            @Override
            public void changed(ObservableValue<? extends Plant> observable, Plant oldValue, Plant newValue) {
                if(newValue != null) {
                    selectedPlant = newValue;
                    description_plant.setText(selectedPlant.description());
                    saveToMyPlant_button.setDisable(false);
                    //ToDo Uncomment if plant.java can load img
                    //Image img = selectedPlant.image();
                    //img_plant.setImage(img);
                } else {
                    selectedPlant = null;
                    description_plant.setText("");
                    saveToMyPlant_button.setDisable(true);
                    Image img = new Image(String.valueOf(PlantsController.class.getResource("placeholder.png")));
                    img_plant.setImage(img);
                }
            }
        });
    }


    /**
     * clears the ListView of entries
     */
    private void clearListView() {
        list_plants.getItems().clear();
    }
}
